@@ -66,6 +66,7 @@ class MSSPlotDriver(metaclass=ABCMeta):
         self.data_access = data_access_object
         self.dataset = None
         self.plot_object = None
+        self.filenames = []
 
     def __del__(self):
         """
@@ -89,6 +90,7 @@ class MSSPlotDriver(metaclass=ABCMeta):
         if len(self.plot_object.required_datafields) == 0:
             logging.debug("no datasets required.")
             self.dataset = None
+            self.filenames = []
             self.init_time = None
             self.fc_time = None
             self.times = np.array([])
@@ -114,13 +116,10 @@ class MSSPlotDriver(metaclass=ABCMeta):
         # Check if a dataset is open and if it contains the requested times.
         # (a dataset will only be open if the used layer has not changed,
         # i.e. the required variables have not changed as well).
-        if self.dataset is not None:
-            logging.debug("checking on open dataset.")
-            if self.init_time == init_time:
-                logging.debug("\tinitialisation time ok (%s).", init_time)
-                if fc_time in self.times:
-                    logging.debug("\tforecast valid time contained (%s).", fc_time)
-                    return
+        if (self.dataset is not None) and (self.init_time == init_time) and (fc_time in self.times):
+            logging.debug("\tinit time correct and forecast valid time contained (%s).", fc_time)
+            if not self.data_access.is_reload_required(self.filenames):
+                return
             logging.debug("need to re-open input files.")
             self.dataset.close()
             self.dataset = None
@@ -129,16 +128,16 @@ class MSSPlotDriver(metaclass=ABCMeta):
         # requested time:
 
         # Create the names of the files containing the required parameters.
-        filenames = []
+        self.filenames = []
         for vartype, var, _ in self.plot_object.required_datafields:
             filename = self.data_access.get_filename(
                 var, vartype, init_time, fc_time, fullpath=True)
-            if filename not in filenames:
-                filenames.append(filename)
+            if filename not in self.filenames:
+                self.filenames.append(filename)
             logging.debug("\tvariable '%s' requires input file '%s'",
                           var, os.path.basename(filename))
 
-        if len(filenames) == 0:
+        if len(self.filenames) == 0:
             raise ValueError("no files found that correspond to the specified "
                              "datafields. Aborting..")
 
@@ -147,7 +146,7 @@ class MSSPlotDriver(metaclass=ABCMeta):
         # Open NetCDF files as one dataset with common dimensions.
         logging.debug("opening datasets.")
         dsKWargs = self.data_access.mfDatasetArgs()
-        dataset = netCDF4tools.MFDatasetCommonDims(filenames, **dsKWargs)
+        dataset = netCDF4tools.MFDatasetCommonDims(self.filenames, **dsKWargs)
 
         # Load and check time dimension. self.dataset will remain None
         # if an Exception is raised here.
@@ -274,7 +273,7 @@ class MSSPlotDriver(metaclass=ABCMeta):
         figsize = figsize if figsize is not None else self.figsize
         noframe = noframe if noframe is not None else self.noframe
         init_time = init_time if init_time is not None else self.init_time
-        valid_time = valid_time if valid_time is not None else self.valid_time
+        valid_time = valid_time if valid_time is not None else self.fc_time
         style = style if style is not None else self.style
         bbox = bbox if bbox is not None else self.bbox
         transparent = transparent if transparent is not None else self.transparent
@@ -389,7 +388,7 @@ class VerticalSectionDriver(MSSPlotDriver):
         noframe = noframe if noframe is not None else self.noframe
         draw_verticals = draw_verticals if draw_verticals else self.draw_verticals
         init_time = init_time if init_time is not None else self.init_time
-        valid_time = valid_time if valid_time is not None else self.valid_time
+        valid_time = valid_time if valid_time is not None else self.fc_time
         style = style if style is not None else self.style
         bbox = bbox if bbox is not None else self.bbox
         vsec_path = vsec_path if vsec_path is not None else self.vsec_path
@@ -592,7 +591,7 @@ class HorizontalSectionDriver(MSSPlotDriver):
         figsize = figsize if figsize is not None else self.figsize
         noframe = noframe if noframe is not None else self.noframe
         init_time = init_time if init_time is not None else self.init_time
-        valid_time = valid_time if valid_time is not None else self.valid_time
+        valid_time = valid_time if valid_time is not None else self.fc_time
         style = style if style is not None else self.style
         bbox = bbox if bbox is not None else self.bbox
         level = level if level is not None else self.level
@@ -707,7 +706,7 @@ class LinearSectionDriver(VerticalSectionDriver):
         """
         plot_object = plot_object if plot_object is not None else self.plot_object
         init_time = init_time if init_time is not None else self.init_time
-        valid_time = valid_time if valid_time is not None else self.valid_time
+        valid_time = valid_time if valid_time is not None else self.fc_time
         bbox = bbox if bbox is not None else self.bbox
         lsec_path = lsec_path if lsec_path is not None else self.lsec_path
         lsec_numpoints = lsec_numpoints if lsec_numpoints is not None else self.lsec_numpoints
@@ -773,10 +772,11 @@ class LinearSectionDriver(VerticalSectionDriver):
         lon_data = lon_data[lon_indices]
         factors = []
 
-        # Make sure air_pressure is the first to be evaluated
+        # Make sure air_pressure is the first to be evaluated if needed
         variables = list(self.data_vars)
-        if variables[0] != "air_pressure":
-            variables.insert(0, variables.pop(variables.index("air_pressure")))
+        if "air_pressure" in self.data_vars:
+            if variables[0] != "air_pressure":
+                variables.insert(0, variables.pop(variables.index("air_pressure")))
 
         for name in variables:
             var = self.data_vars[name]
@@ -796,9 +796,10 @@ class LinearSectionDriver(VerticalSectionDriver):
             cross_section = utils.interpolate_vertsec(var_data, self.lat_data, lon_data, self.lats, self.lons)
             # Create vertical interpolation factors and indices for subsequent variables
             # TODO: Improve performance for this interpolation in general
-            if name == "air_pressure":
+            if len(factors) == 0:
                 for index_lonlat, alt in enumerate(self.alts):
-                    pressures = cross_section[:, index_lonlat]
+                    pressures = cross_section[:, index_lonlat] if name == "air_pressure" \
+                        else self.vert_data[::-self.vert_order] * (100 if self.vert_units.lower() == "hpa" else 1)
                     closest = 0
                     direction = 1
                     for index_altitude, pressure in enumerate(pressures):
@@ -818,6 +819,7 @@ class LinearSectionDriver(VerticalSectionDriver):
                             [[closest, 1 - (abs(pressures[closest] - alt) / distance)],
                              [next_closest, 1 - (abs(pressures[next_closest] - alt) / distance)]])
 
+            # Interpolate with the previously calculated pressure indices and factors
             for index in range(len(self.alts)):
                 cur_factor = factors[index]
                 value = cross_section[cur_factor[0][0], index] * cur_factor[0][1] + \
